from pathlib import Path

from coast import COASTResults
from coast.util.input import loader
from coast.align.blast import BlastBatch
from coast.data.alignment_results import AlignmentResults
from coast.util.workspace import generate_folder
from coast.coast_var import output_format
from Bio import SeqIO


def compare_handler(args):
    """Handles args coming from the 'coast compare' subcommand"""
    compare(queries=args.queries,
            keywords=args.keywords,
            output_path=args.out,
            containerization=args.containerization,
            image=args.image,
            task=args.task,
            matrix=args.matrix,
            evalue=args.evalue,
            gapopen=args.gapopen,
            gapextend=args.gapextend,
            gen_ws=args.gen_ws,
            threads=args.threads)


def compare(queries,
            keywords,
            engine="blast",
            output_path=None,
            task=None,
            matrix=None,
            evalue=None,
            gapextend=None,
            gapopen=None,
            containerization=None,
            image=None,
            threads=None,
            gen_ws=None):
    """ COAST comparison between all against all proteomes
    """
    # Loads all sequences, confirm if it is more than one
    sequences = {}
    organism_dict = {}
    for query in queries:
        query_path = Path(query)
        content = loader(str(query_path.resolve()), keywords)
        organism_dict[query_path.stem] = list(content.keys())
        sequences.update(loader(str(query_path.resolve()), keywords))
    wd = Path(output_path)

    if gen_ws:
        wd = generate_folder(str(wd.resolve()), "COMPARE")
    SeqIO.write(sequences.values(), wd.joinpath("protein.faa").resolve(), "fasta")

    if engine == "blast":
        aln = BlastBatch(query=wd.joinpath("protein.faa").resolve(),
                         out="blast_results.tab",
                         subject=wd.joinpath("protein.faa").resolve(),
                         task=task,
                         matrix=matrix,
                         evalue=evalue,
                         gapextend=gapextend,
                         gapopen=gapopen,
                         outfmt=f'"6 {output_format["blast-short"]}"')
    else:
        raise Exception("Engine is not available")
    # if engine == "diamond":
    #     aln = DiamondBatch(query=wd.joinpath("protein.faa").resolve(),
    #                        out="blast_results.tab",
    #                        db=db,
    #                        taxonlist=taxonlist,
    #                        taxidlist=taxidlist,
    #                        sensitivity=sensitivity,
    #                        matrix=matrix,
    #                        evalue=evalue,
    #                        gapextend=gapextend,
    #                        gapopen=gapopen)

    if image and aln:
        aln.run_container(containerization=containerization, image=image)
    # elif web:
    #     aln.run_merlin(email=email, address=web)
    else:
        aln.run_local(threads=threads)

    # save coast formatted file
    a = AlignmentResults.from_file(wd.joinpath("blast_results.tab"), header=output_format["blast-short"].split())
    b = COASTResults.from_alignment_results(a, organism_dict)
    b.save_file(wd.joinpath("coast_aln.tab"))
