import os
from pathlib import Path

from coast.sm import SimilarityMatrix
from coast.sm_complex import ReciprocalSimilarityMatrix
from coast.util.input import loader


def report_handler(args):
    """Handles args coming from the 'coast report' subcommand into the reporting function"""
    for arg in args.outfmt:
        if arg not in ["a", "b", "r"]:
            raise Exception(f"The output format '{arg}' is not in the authorized list b/a/r.")
    report(queries=args.queries,
           keywords=args.keywords,
           results_path=args.results,
           output_path=args.out,
           out_fmt=args.outfmt,
           coverage_thres=args.cov,
           identity_thres=args.id,
           aai_thres=args.aai,
           reciprocal=args.reciprocal,
           hypothetical=args.hypothetical,
           verbose=args.verbose)


def report(queries,
           keywords,
           results_path,
           output_path,
           out_fmt,
           coverage_thres,
           identity_thres,
           aai_thres,
           reciprocal: bool = False,
           hypothetical: bool = False,
           verbose: bool = False):
    """
    Reporting function that generates 
    """
    sequences = {}
    organism_dict = {}

    for query in queries:
        query_path = Path(query)
        content = loader(str(query_path.resolve()), key=keywords, hypothetical=hypothetical)
        organism_dict[query_path.stem] = list(content.keys())
        sequences.update(loader(str(query_path.resolve())))

    if reciprocal:
        job = ReciprocalSimilarityMatrix.from_results(sub_sequences=sequences, results_path=results_path, min_cov=coverage_thres, min_id=identity_thres, organisms=organism_dict)
    elif not reciprocal and len(queries) > 1:
        raise Exception(
            f"Non reciprocal best hits, can only process 1 query. You have provided {len(queries)} queries.")
    else:
        job = SimilarityMatrix.from_results(sub_sequences=sequences, results_path=results_path, min_cov=coverage_thres, min_id=identity_thres, organisms=organism_dict)


    if "b" in out_fmt:
        job.tabular_sm[(job.tabular_sm['pident'] != 0)].to_csv(path_or_buf=os.path.join(output_path, "bh_results.tab"),
                                                               sep="\t", index=False)
    if "a" in out_fmt:
        job.tabular_agr_sm.to_csv(path_or_buf=os.path.join(output_path, "coast_results.tab"), sep="\t", index=False)
    if "r" in out_fmt:
        job.html_agr_sm(output_path=os.path.join(output_path, "coast_report.html"))
    if verbose:
        job.print_agr_sm(min_aai=aai_thres)
