
def interactive_heatmap(source_data, row_labels, col_labels, title):
    """
    data: numpy array
    row_labels: list
    col_labels: list
    """
    from bokeh.plotting import figure, show
    from bokeh.models import (BasicTicker, ColorBar, ColumnDataSource, LinearColorMapper, PrintfTickFormatter)
    from bokeh.transform import transform
    from bokeh.embed import components

    source = ColumnDataSource(source_data)
    colors = [
        "#ffffff",
        "#e8f1fd",
        "#d1e3fb",
        "#bad5f9",
        "#a3c7f7",
        "#8bb8f5",
        "#74aaf3",
        "#5d9cf1",
        "#468eef",
        "#2f80ed"
    ]
    mapper = LinearColorMapper(palette=colors, low=0.0, high=100.0)

    width = 1050
    # height = int(round(width/(20*len(col_labels))*len(row_labels), 0))
    height = 1200
    plot = figure(title=title, x_axis_location="above", width=width, height=height, tools="hover,save",
                  x_range=col_labels,
                  y_range=row_labels, tooltips=[('Query Sequence', '@qseqid'), ('Match Staxid', '@staxids'),
                                                ('Match Identity', '@pident %'), ('Match Coverage', '@qcovs %')])
    plot.rect('qseqid', 'staxids', 1, 1, source=source,
              fill_color=transform('pident', mapper), line_color=None,
              hover_line_color='black')

    color_bar = ColorBar(color_mapper=mapper,
                         ticker=BasicTicker(desired_num_ticks=len(colors)),
                         formatter=PrintfTickFormatter(format="%d%%"))
    plot.add_layout(color_bar, 'right')

    plot.axis.axis_line_color = None
    plot.axis.major_tick_line_color = None
    plot.axis.major_label_text_font_size = "10px"
    plot.axis.major_label_standoff = 0
    plot.xaxis.major_label_orientation = 1.0

    script, div = components(plot)
    return script, div


def interactive_correlation_matrix(source_data, row_labels, col_labels, title, index="aaibd"):
    """
    data: numpy array
    row_labels: list
    col_labels: list
    """
    from bokeh.plotting import figure, show
    from bokeh.models import (BasicTicker, ColorBar, ColumnDataSource, LinearColorMapper, PrintfTickFormatter)
    from bokeh.transform import transform
    from bokeh.embed import components

    source = ColumnDataSource(source_data)
    colors = [
        "#ffffff",
        "#e8f1fd",
        "#d1e3fb",
        "#bad5f9",
        "#a3c7f7",
        "#8bb8f5",
        "#74aaf3",
        "#5d9cf1",
        "#468eef",
        "#2f80ed"
    ]
    mapper = LinearColorMapper(palette=colors, low=0.0, high=100.0)

    width = 1300
    # height = int(round(width/(20*len(col_labels))*len(row_labels), 0))
    height = 1200
    plot = figure(title=title, x_axis_location="above", width=width, height=height, tools="hover,save",
                  x_range=col_labels,
                  y_range=row_labels, tooltips=[('Query Organism', '@qname'), ('Match Organism', '@sname'),
                                                ('Match AAIC', '@aaic %'),('Match AAIBD', '@aaibd %'), ('Match Percent', '@pmatch %')])
    plot.rect('qname', 'sname', 1, 1, source=source,
              fill_color=transform(index, mapper), line_color=None,
              hover_line_color='black')

    color_bar = ColorBar(color_mapper=mapper,
                         ticker=BasicTicker(desired_num_ticks=len(colors)),
                         formatter=PrintfTickFormatter(format="%d%%"))
    plot.add_layout(color_bar, 'right')

    plot.axis.axis_line_color = None
    plot.axis.major_tick_line_color = None
    plot.axis.major_label_text_font_size = "10px"
    plot.axis.major_label_standoff = 0
    plot.xaxis.major_label_orientation = 1.0

    script, div = components(plot)
    return script, div
