from Bio import SeqIO
from pathlib import Path

from coast.util.input import loader


def genbank_to_blast_tax(output_folder, *files):
    output_folder = Path(output_folder)

    sequences = {}
    lines = []

    for file in files:
        file = Path(file)
        content = SeqIO.read(file, format="genbank")
        taxid = content.features[000].qualifiers["db_xref"][0].split(":")[1]
        for feature in content.features:
            if feature.type == "CDS":
                lines.append(f'{feature.qualifiers["protein_id"][0]} {taxid}\n')
        sequences.update(loader(file, "CDS"))

    with open(output_folder / "tax.txt", "w") as output:
        output.writelines(lines)
    SeqIO.write(sequences.values(), output_folder / "sequences.faa", "fasta")


def genbank_to_diamond_tax(output_folder, *files):
    """
    https://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/README
    """
    output_folder = Path(output_folder)

    sequences = {}
    lines = ["accession\taccession.version\ttaxid\tgi\n"]

    for file in files:
        file = Path(file)
        content = SeqIO.read(file, format="genbank")
        taxid = content.features[000].qualifiers["db_xref"][0].split(":")[1]
        for feature in content.features:
            if feature.type == "CDS":
                lines.append(
                    f'{feature.qualifiers["protein_id"][0].split(".")[0]}\t{feature.qualifiers["protein_id"][0]}\t{taxid}\tna\n')
        sequences.update(loader(file, "CDS"))

    with open(output_folder / "tax.txt", "w") as output:
        output.writelines(lines)
    SeqIO.write(sequences.values(), output_folder / "sequences.faa", "fasta")


if __name__ == '__main__':
    genbank_to_blast_tax("../../../sample_data/blast_db/source", "../../../sample_data/MN844877.gb",
                         "../../../sample_data/MN908947.3.gb", "../../../sample_data/NC_016071.gb")
    genbank_to_diamond_tax("../../../sample_data/diamond_db/source", "../../../sample_data/MN844877.gb",
                           "../../../sample_data/MN908947.3.gb", "../../../sample_data/NC_016071.gb")
